/******************************************************************************
 * File: jMileage.c
 * Application: jMileage
 * Revision 1.5
 * Copyright (c) 2001 Jason Shah, jasonshah@yahoo.com
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
*******************************************************************************/



// TODO:
// -1. More Details screen
//    -1a. This fillup
//         -a1. Total cost of fillup
//         -a2. Cost per unit distance of this fillup
//    -1b. All fillups
//         -b1. Avg cost per unit distance
//         -b2. Cost per day/month/year
//         b3. Min/max/avg fuel efficiency
// -2. cost/unit qty and notes
//    -2a. Conversion from old to new
//    -2b. Preferences update for fields on save prefs
//    -2c. Storage of cost/qty and notes on add new
//    -2d. Edit form update
//    -2e. Edit form logic update to store notes and cost/qty
//    -2f. FillCarListArray
// -3. Error check on field/db resize
// -4. Increased precision on refillF
// -5. More information on main screen
// -6. Confirm delete
// -7. Change "Input Error" to an ALERT
// -8. Handle scroll down events in list
// -9. Fix precision in list for output mileage (i.e. no decimal places)
// 10. Export to Memo; Menu and Code


#include <PalmOS.h>
#include <FloatMgr.h>
#include "gccfloat.h"
#include "jMileage.h"
#include "jMileageResource.h"

#ifdef __GNUC__
#include <Callback.h>
#endif



// ***** Static string definitions
static Char* EXPORT_TITLE_HEADER_STR = "jMileage Export\nDate,Start,End,Qty,Cost/Qty,Note\n";



// ***** StartApplication *****
// Called when application is starting
static Err StartApplication()
{
	Err error = 0;
	FormType *form;
	FieldType *field;
	MemHandle resourceH;
	MemHandle resource2H;
	Char *resource;
	Char *MPGString;
	Char *AMPGString;
	UInt16 prefSize;
	UInt16 i;
	Char *str;
	jMileageRecordType *jMRTArray = NULL;

	// load preferences
	prefSize = sizeof(jMileagePrefType);
	prefsRetrieved = false;
	MemSet(&prefs, sizeof(prefs), 0);
	if (PrefGetAppPreferences(creatorID, 0, &prefs, &prefSize, true) !=
		noPreferenceFound)
	{
		prefsRetrieved = true;
	}
	else
	{
		StrCopy(prefs.car1, "Auto 1");
		StrCopy(prefs.car2, "Auto 2");
		StrCopy(prefs.car3, "Auto 3");
		prefs.startF[0].f = prefs.startF[1].f = prefs.startF[2].f = -1;
		prefs.endF[0].f = prefs.endF[1].f = prefs.endF[2].f = -1;
		prefs.refillF[0].f = prefs.refillF[1].f = prefs.refillF[2].f = -1;
		prefs.cpuF[0].f = prefs.cpuF[1].f = prefs.cpuF[2].f = -1;
		prefs.carChosen = 1;
		prefs.prefLabelType = MPG;
		prefs.prefDateType = MSDSY;
		prefs.versionNum = 105;
	}

	// create database if it doesn't exist
	dB = DmOpenDatabaseByTypeCreator(dBType, creatorID, dmModeReadWrite);
	if (!dB)
	{
		UInt16 cardNo;
		LocalID dbID, appInfoID;
		UInt16 attributes;
		MemHandle h;
		jMileageAppInfoType *appInfo;

		// create database, failing if error
		error = DmCreateDatabase(0, dBName, creatorID, dBType, false);
		if (error) return error;
		dB = DmOpenDatabaseByTypeCreator(dBType, creatorID, dmModeReadWrite);
		if (!dB) return DmGetLastErr();

		// set backup attribute and init app info block
		DmOpenDatabaseInfo(dB, &dbID, NULL, NULL, &cardNo, NULL);
		DmDatabaseInfo(cardNo, dbID, NULL, &attributes, NULL, NULL, NULL, NULL,
			NULL, &appInfoID, NULL, NULL, NULL);
		attributes |= dmHdrAttrBackup;
		if (appInfoID==NULL)
		{
			h = DmNewHandle(dB, sizeof(jMileageAppInfoType));
			if (!h) error = dmErrMemError;
			appInfoID = MemHandleToLocalID(h);
			DmSetDatabaseInfo(cardNo, dbID, NULL, NULL, NULL, NULL, NULL,
				NULL, NULL, &appInfoID, NULL, NULL, NULL); // set appinfo id
		}
		appInfo = MemLocalIDToLockedPtr(appInfoID, cardNo);
		DmSet(appInfo, 0, sizeof(jMileageAppInfoType), 0); // clear app info blk

		// initialize categories
		CategoryInitialize((AppInfoType *)appInfo, CategoryAppInfoStr);

		MemPtrUnlock(appInfo);
		DmSetDatabaseInfo(cardNo, dbID, NULL, &attributes, NULL, NULL, NULL, 
			NULL, NULL, NULL, NULL, NULL, NULL); // set attributes
		if (error) // if an error, close and delete empty db
		{
			DmCloseDatabase(dB);
			DmDeleteDatabase(cardNo, dbID);
			return error;
		}
	}

	// update database if necessary
	oldVersionNum = 105;
	if (prefs.versionNum < 105 || prefs.versionNum > 105)
	{
		prefs.versionNum = 105;
		UpdateToV105();
	}

	// initialize MPG and AMPG labels
	InitLabels();

	// fill the car list with cars from prefs.carChosen
	FillCarListArray();

	// load main form
	FrmGotoForm(jMileageForm);

	return 0;
}






// ***** StopApplication *****
// Called when application is being shut down
static void StopApplication()
{
	Err error;
	jMileageRecordType *jMRTArray;
	UInt16 i;

	// free MPGStringH and AMPGStringH
	MemHandleFree(MPGStringH);
	MemHandleFree(AMPGStringH);

	// close the database
	DmCloseDatabase(dB);

	// free jMRTArrayH
	MemHandleFree(jMRTArrayH);

	// close all forms
	FrmCloseAllForms();
}










// ***** jMileageFormEventHandler *****
// Handle events for jMileageForm form
Boolean jMileageFormEventHandler(EventPtr event)
{
	Boolean handled = false;
	FormType *form;

#ifdef __GNUC__
CALLBACK_PROLOGUE;
#endif

	form = FrmGetActiveForm();

	switch (event->eType)
	{
		// frmOpenEvent: the main form has been opened
		case frmOpenEvent:
		{
			FieldType *field;
			ListType *list;
			EventType newEvent;
			ControlType *popupTrig;
			MemHandle memH, memH2;
			Char *str;
			Char tmpStr[21];
			UInt16 i;
			RectangleType rect;

			// fill start with saved preferences
			memH = MemHandleNew(13);
			str = MemHandleLock(memH);
			if (prefs.startF[prefs.carChosen-1].f >= 0)
				PrintF(str, prefs.startF[prefs.carChosen-1].f, 12, 1);
			else
				MemSet(str, MemHandleSize(memH), 0);
			MemHandleUnlock(memH);
			field = FrmGetObjectPtr(form, 
				FrmGetObjectIndex(form, jMileageStartField));
			memH2 = FldGetTextHandle(field);
			FldSetTextHandle(field, memH);
			FldDrawField(field);
			if (memH2) MemHandleFree(memH2);

			// fill end with saved preferences
			memH = MemHandleNew(13);
			str = MemHandleLock(memH);
			if (prefs.endF[prefs.carChosen-1].f >= 0)
				PrintF(str, prefs.endF[prefs.carChosen-1].f, 12, 1);
			else
				MemSet(str, MemHandleSize(memH), 0);
			MemHandleUnlock(memH);
			field = FrmGetObjectPtr(form, 
				FrmGetObjectIndex(form, jMileageEndField));
			memH2 = FldGetTextHandle(field);
			FldSetTextHandle(field, memH);
			FldDrawField(field);
			if (memH2) MemHandleFree(memH2);

			// fill refill with saved preferences
			memH = MemHandleNew(13);
			str = MemHandleLock(memH);
			if (prefs.refillF[prefs.carChosen-1].f >= 0)
				PrintF(str, prefs.refillF[prefs.carChosen-1].f, 12, 3);
			else
				MemSet(str, MemHandleSize(memH), 0);
			MemHandleUnlock(memH);
			field = FrmGetObjectPtr(form, 
				FrmGetObjectIndex(form, jMileageRefillField));
			memH2 = FldGetTextHandle(field);
			FldSetTextHandle(field, memH);
			FldDrawField(field);
			if (memH2) MemHandleFree(memH2);

			// fill cost per unit with saved preferences
			memH = MemHandleNew(7);
			str = MemHandleLock(memH);
			if (prefs.cpuF[prefs.carChosen-1].f >= 0)
				PrintF(str, prefs.cpuF[prefs.carChosen-1].f, 5, 3);
			else
				MemSet(str, MemHandleSize(memH), 0);
			MemHandleUnlock(memH);
			field = FrmGetObjectPtr(form, 
				FrmGetObjectIndex(form, jMileageCPUField));
			memH2 = FldGetTextHandle(field);
			FldSetTextHandle(field, memH);
			FldDrawField(field);
			if (memH2) MemHandleFree(memH2);

			// fill notes with saved preferences
			memH = MemHandleNew(31);
			str = MemHandleLock(memH);
			MemSet(str, MemHandleSize(memH), 0);
			StrCopy(str, prefs.notes[prefs.carChosen-1]);
			MemHandleUnlock(memH);
			field = FrmGetObjectPtr(form, 
				FrmGetObjectIndex(form, jMileageNoteField));
			memH2 = FldGetTextHandle(field);
			FldSetTextHandle(field, memH);
			FldDrawField(field);
			if (memH2) MemHandleFree(memH2);

			// fill list with entries in jMRTArrayH
			list = FrmGetObjectPtr(form,
				FrmGetObjectIndex(form, jMileageOldEntryList));
			LstSetDrawFunction(list, (ListDrawDataFuncPtr)ListDrawFn);
			LstSetListChoices(list, NULL, jMRTArrayNum);
			LstSetSelection(list, -1);
			LstMakeItemVisible(list, jMRTArrayNum-1); // scroll to bottom
			LstDrawList(list);

			// populate car list and set ptr
			list = FrmGetObjectPtr(form,
				FrmGetObjectIndex(form, jMileageCarList));
			carList[0] = prefs.car1;
			carList[1] = prefs.car2;
			carList[2] = prefs.car3;
			carList[3] = carListLastOption; // Edit Categories
			LstSetListChoices(list, carList, 4);
			LstSetSelection(list, prefs.carChosen-1); // set which is chosen
			popupTrig = FrmGetObjectPtr(form, 
				FrmGetObjectIndex(form, jMileageCarPopup));
			switch(prefs.carChosen)
			{
				case 1:
					StrCopy(tmpStr, prefs.car1);
					break;
				case 2:
					StrCopy(tmpStr, prefs.car2);
					break;
				case 3:
					StrCopy(tmpStr, prefs.car3);
					break;
			}
			CtlSetLabel(popupTrig, tmpStr);

			// set the appropriate focus
			if (prefs.whichHasFocus[prefs.carChosen-1]==1)
				FrmSetFocus(form, FrmGetObjectIndex(form, jMileageStartField));
			else if (prefs.whichHasFocus[prefs.carChosen-1]==2)
				FrmSetFocus(form, FrmGetObjectIndex(form, jMileageEndField));
			else if (prefs.whichHasFocus[prefs.carChosen-1]==3)
				FrmSetFocus(form, FrmGetObjectIndex(form, jMileageRefillField));
			else if (prefs.whichHasFocus[prefs.carChosen-1]==4)
				FrmSetFocus(form, FrmGetObjectIndex(form, jMileageCPUField));
			else if (prefs.whichHasFocus[prefs.carChosen-1]==5)
				FrmSetFocus(form, FrmGetObjectIndex(form, jMileageNoteField));

			FrmSetMenu(form, MainMenuBar);
			FrmDrawForm(form);

			// clear the bottom area
			ClearAllMPG();
			
			// draw avg MPG (AFTER FrmDrawForm)
			DrawAMPG();

			handled = true;
			break;
		}

		// frmCloseEvent: when app exited
		case frmCloseEvent:
		{
			UInt16 focusOn;

			// set whichHasFocus
			focusOn = FrmGetFocus(form);
			if (focusOn == FrmGetObjectIndex(form, jMileageStartField))
				prefs.whichHasFocus[prefs.carChosen-1] = 1;
			else if (focusOn == FrmGetObjectIndex(form, jMileageEndField))
				prefs.whichHasFocus[prefs.carChosen-1] = 2;
			else if (focusOn == FrmGetObjectIndex(form, jMileageRefillField))
				prefs.whichHasFocus[prefs.carChosen-1] = 3;
			else if (focusOn == FrmGetObjectIndex(form, jMileageCPUField))
				prefs.whichHasFocus[prefs.carChosen-1] = 4;
			else if (focusOn == FrmGetObjectIndex(form, jMileageNoteField))
				prefs.whichHasFocus[prefs.carChosen-1] = 5;

			// save fields to preferences
			SaveFieldsToPrefs(form);

			// save preferences
			PrefSetAppPreferences(creatorID, 0, 1, &prefs,
				sizeof(prefs), true);

			break;
		}

		// frmUpdateEvent: an update is called
		case frmUpdateEvent:
		{
			RectangleType rect;
			ListType *list;
			ControlType *popupTrig;

			/*
			  XXX For some reason, even after FrmDrawForm, crap from the Edit
			  XXX Categories form remains on the jMileage form in the draw area
			  XXX at the bottom.  This clears that area out and redraws it.
			*/

			// Clear out the entire form
			rect.topLeft.x = 0;
			rect.topLeft.y = 0;
			rect.extent.x = 160;
			rect.extent.y = 160;
			WinEraseRectangle(&rect, 0);
			// Draw the form; this cleans out all PalmOS-defined objects
			FrmDrawForm(form);
			// Draw the average miles per gallon
			DrawAMPG();
			// Set the popup label
			popupTrig = FrmGetObjectPtr(form, 
				FrmGetObjectIndex(form, jMileageCarPopup));
			switch(prefs.carChosen)
			{
				case 1:
					CtlSetLabel(popupTrig, prefs.car1);
					break;
				case 2:
					CtlSetLabel(popupTrig, prefs.car2);
					break;
				case 3:
					CtlSetLabel(popupTrig, prefs.car3);
					break;
			}

			// Simulate a selection on the entry selected (if any)
			list = FrmGetObjectPtr(form,
				FrmGetObjectIndex(form, jMileageOldEntryList));
			if (LstGetSelection(list)!=noListSelection)
			{
				EventType newEvent;
				newEvent.eType = lstSelectEvent;
				newEvent.data.lstSelect.listID=jMileageOldEntryList;
				newEvent.data.lstSelect.pList = list;
				newEvent.data.lstSelect.selection = LstGetSelection(list);
				EvtAddEventToQueue(&newEvent);
			}
			break;
		}

		// ctlSelectEvent: something has been selected
		case ctlSelectEvent:
		{
			switch (event->data.ctlSelect.controlID)
			{
				// save btn
				case jMileageSaveBtn:
				{
					FieldType *field;
					ListType *list;
					float resultF;
					Char *str;
					Char tmpStr[21];
					UInt8 error = 0;
					UInt16 focusOn;
				

					// erase output area
					ClearMPG();

					// save the fields (start, end, refill) to prefs
					error = SaveFieldsToPrefs(form);

					// set whichHasFocus
					focusOn = FrmGetFocus(form);
					if (focusOn == FrmGetObjectIndex(form, jMileageStartField))
						prefs.whichHasFocus[prefs.carChosen-1] = 1;
					else if (focusOn == FrmGetObjectIndex(form, 
						jMileageEndField))
						prefs.whichHasFocus[prefs.carChosen-1] = 2;
					else if (focusOn == FrmGetObjectIndex(form, 
						jMileageRefillField))
						prefs.whichHasFocus[prefs.carChosen-1] = 3;
					else if (focusOn == FrmGetObjectIndex(form, 
						jMileageCPUField))
						prefs.whichHasFocus[prefs.carChosen-1] = 4;
					else if (focusOn == FrmGetObjectIndex(form, 
						jMileageNoteField))
						prefs.whichHasFocus[prefs.carChosen-1] = 5;

					// set carChosen
					list = FrmGetObjectPtr(form,
						FrmGetObjectIndex(form, jMileageCarList));
					prefs.carChosen = LstGetSelection(list)+1;

					if (!error)
					{
						Err err;
						FontID	oldFont;
						Char *str;
						jMileageRecordType *jMRTArray;
						jMileageRecordType *jMRT;
						EventType newEvent;
						ListType *list = FrmGetObjectPtr(form,
							FrmGetObjectIndex(form, jMileageOldEntryList));
						MemHandle tmpMemH1, tmpMemH2;
						FieldType *field2;
						UInt16 dBPos;
						MemHandle dBRecordH;
						jMileageRecordType *dBRecordP;
						UInt16 attributes;

						// add entry to jMRTArray 
						jMRTArrayNum += 1;
						err = MemHandleResize(jMRTArrayH, 
							MemHandleSize(jMRTArrayH)
							+sizeof(jMileageRecordType));
						if (err==memErrInvalidParam)
							ErrDisplay("Invalid Parameter to resize memory");
						else if (err==memErrNotEnoughSpace)
						{
							ErrAlert(ErrNoSpaceAlert);
							FrmCloseAllForms();
						}
						else if (err==memErrChunkLocked)
							ErrDisplay("Memory chunk locked on resize mem");

						jMRTArray = MemHandleLock(jMRTArrayH);
						jMRT = jMRTArray+(jMRTArrayNum-1);
						jMRT->startF = prefs.startF[prefs.carChosen-1].f;
						jMRT->endF = prefs.endF[prefs.carChosen-1].f;
						jMRT->refillF = prefs.refillF[prefs.carChosen-1].f;
						jMRT->date = TimGetSeconds();
						jMRT->cpuF = prefs.cpuF[prefs.carChosen-1].f;
						StrCopy(jMRT->notes, prefs.notes[prefs.carChosen-1]);

						// save entry into database with appropriate category
						AddToDB(jMRT);

						// insertion sort jMRTArray
						SysInsertionSort(jMRTArray, jMRTArrayNum, 
							sizeof(jMileageRecordType), 
							(const CmpFuncPtr) &(jMRCompareFunc), 0);

						MemHandleUnlock(jMRTArrayH);

						// increment the form's list's size
						LstSetListChoices(list, NULL, jMRTArrayNum);
						LstSetSelection(list, jMRTArrayNum-1);
						LstMakeItemVisible(list, jMRTArrayNum-1);
						LstDrawList(list);

						// move End field to Start
						field = FrmGetObjectPtr(form,
							FrmGetObjectIndex(form, jMileageStartField));
						field2 = FrmGetObjectPtr(form,
							FrmGetObjectIndex(form, jMileageEndField));
						tmpMemH1 = FldGetTextHandle(field2);
						FldSetTextHandle(field2, NULL);
						tmpMemH2 = FldGetTextHandle(field);
						FldSetTextHandle(field, tmpMemH1);
						FldDrawField(field);
						// clear out End field
						str = MemHandleLock(tmpMemH2);
						MemSet(str, MemHandleSize(tmpMemH2), 0);
						MemHandleUnlock(tmpMemH2);
						FldSetTextHandle(field2, tmpMemH2);
						FldDrawField(field2);
						// clear out Refill field
						field = FrmGetObjectPtr(form, 
							FrmGetObjectIndex(form, jMileageRefillField));
						tmpMemH1 = FldGetTextHandle(field);
						FldSetTextHandle(field, NULL);
						str = MemHandleLock(tmpMemH1);
						MemSet(str, MemHandleSize(tmpMemH1), 0);
						MemHandleUnlock(tmpMemH1);
						FldSetTextHandle(field, tmpMemH1);
						FldDrawField(field);
						// clear out Note field
						field = FrmGetObjectPtr(form, 
							FrmGetObjectIndex(form, jMileageNoteField));
						tmpMemH1 = FldGetTextHandle(field);
						FldSetTextHandle(field, NULL);
						str = MemHandleLock(tmpMemH1);
						MemSet(str, MemHandleSize(tmpMemH1), 0);
						MemHandleUnlock(tmpMemH1);
						FldSetTextHandle(field, tmpMemH1);
						FldDrawField(field);

						// set the form's focus to Start field
						FrmSetFocus(form, 
							FrmGetObjectIndex(form, jMileageStartField));

						// draw avg MPG field
						DrawAMPG();

						// change prefs to reflect change in fields
						prefs.startF[prefs.carChosen-1].f 
							= prefs.endF[prefs.carChosen-1].f;
						prefs.endF[prefs.carChosen-1].f = -1;
						prefs.refillF[prefs.carChosen-1].f = -1;
						prefs.notes[prefs.carChosen-1][0] = 0;
						prefs.whichHasFocus[prefs.carChosen-1] = 2;

						// simulate a selection on the last (newest) entry
						endFSetFocus = true;
						newEvent.eType = lstSelectEvent;
						newEvent.data.lstSelect.listID=jMileageOldEntryList;
						newEvent.data.lstSelect.pList = list;
						newEvent.data.lstSelect.selection = jMRTArrayNum-1;
						EvtAddEventToQueue(&newEvent);
					}
					else
					{
						if (error > 10)
						{
							FontID oldFont;
							Char tmpStr[10];
							RectangleType rect;

							// Alert if invalid
							FrmAlert(InvalidInputAlert);
						}
					}

					handled = true;
					break;
				}

				// clear btn
				case jMileageClearBtn:
				{
					FieldType *field;
					MemHandle textH;
					Char *text, *str;

					// erase output area
					ClearMPG();

					// erase start field
					field = FrmGetObjectPtr(form, 
						FrmGetObjectIndex(form, jMileageStartField));
					textH = FldGetTextHandle(field);
					if (textH)
					{
						FldSetTextHandle(field, NULL);
						text = MemHandleLock(textH);
						MemSet(text, MemHandleSize(textH), 0);
						MemHandleUnlock(textH);
						FldSetTextHandle(field, textH);
						FldDrawField(field);
					}
					// erase end field
					field = FrmGetObjectPtr(form, 
						FrmGetObjectIndex(form, jMileageEndField));
					textH = FldGetTextHandle(field);
					if (textH)
					{
						FldSetTextHandle(field, NULL);
						text = MemHandleLock(textH);
						MemSet(text, MemHandleSize(textH), 0);
						MemHandleUnlock(textH);
						FldSetTextHandle(field, textH);
						FldDrawField(field);
					}
					// erase refill field
					field = FrmGetObjectPtr(form, 
						FrmGetObjectIndex(form, jMileageRefillField));
					textH = FldGetTextHandle(field);
					if (textH)
					{
						FldSetTextHandle(field, NULL);
						text = MemHandleLock(textH);
						MemSet(text, MemHandleSize(textH), 0);
						MemHandleUnlock(textH);
						FldSetTextHandle(field, textH);
						FldDrawField(field);
					}
					// erase cost per unit field
					field = FrmGetObjectPtr(form, 
						FrmGetObjectIndex(form, jMileageCPUField));
					textH = FldGetTextHandle(field);
					if (textH)
					{
						FldSetTextHandle(field, NULL);
						text = MemHandleLock(textH);
						MemSet(text, MemHandleSize(textH), 0);
						MemHandleUnlock(textH);
						FldSetTextHandle(field, textH);
						FldDrawField(field);
					}
					// erase notes field
					field = FrmGetObjectPtr(form, 
						FrmGetObjectIndex(form, jMileageNoteField));
					textH = FldGetTextHandle(field);
					if (textH)
					{
						FldSetTextHandle(field, NULL);
						text = MemHandleLock(textH);
						MemSet(text, MemHandleSize(textH), 0);
						MemHandleUnlock(textH);
						FldSetTextHandle(field, textH);
						FldDrawField(field);
					}

					// set all prefs to -1
					prefs.startF[prefs.carChosen-1].f = -1;
					prefs.endF[prefs.carChosen-1].f = -1;
					prefs.refillF[prefs.carChosen-1].f = -1;
					prefs.cpuF[prefs.carChosen-1].f = -1;
					prefs.notes[prefs.carChosen-1][0] = 0;
					prefs.whichHasFocus[prefs.carChosen-1] = 1;

					// set focus
					FrmSetFocus(form, 
						FrmGetObjectIndex(form, jMileageStartField));

					handled = true;
					break;
				}

				default:
					break;
			}
			break;
		}

		// lstSelectEvent: when a list item is selected
		case lstSelectEvent:
		{
			switch (event->data.lstSelect.listID)
			{
				case jMileageOldEntryList:
				{
					jMileageRecordType *jMRTArray;
					jMileageRecordType *jMRT;
					float resultF;
					Char *str;
					Char tmpStr[21];
					FontID oldFont;

					// IF endFSetFocus, set focus on end field
					if (endFSetFocus)
					{
						FrmSetFocus(form, 
							FrmGetObjectIndex(form, jMileageEndField));
						endFSetFocus = false;
					}

					// erase output area
					ClearMPG();

					// draw MPG label and data
					DrawMPG(event->data.lstSelect.selection);
					
					MemHandleUnlock(jMRTArrayH);
					handled = true;
					break;
				}
			}
			break;
		}

		// popSelectEvent: when the popup list was selected
		case popSelectEvent:
		{
			switch (event->data.popSelect.controlID)
			{
				case jMileageCarPopup:
				{
					ListType *list = event->data.popSelect.listP;
					EventType newEvent;
					Int16 selection = event->data.popSelect.selection;
					Int16 priorSelection = event->data.popSelect.priorSelection;
					UInt16 focusOn;

					// if one of first three list items (a car),
					if (selection >= 0 && selection < 3)
					{
						// determine if the car is different than the one open
						if (selection != priorSelection)
						{
							// if different, close this car and open other car

							// set whichHasFocus for this form before leaving it
							focusOn = FrmGetFocus(form);
							if (focusOn == 
								FrmGetObjectIndex(form, jMileageStartField))
								prefs.whichHasFocus[prefs.carChosen-1] = 1;
							else if (focusOn == 
								FrmGetObjectIndex(form, jMileageEndField))
								prefs.whichHasFocus[prefs.carChosen-1] = 2;
							else if (focusOn == 
								FrmGetObjectIndex(form, jMileageRefillField))
								prefs.whichHasFocus[prefs.carChosen-1] = 3;
							else if (focusOn == 
								FrmGetObjectIndex(form, jMileageCPUField))
								prefs.whichHasFocus[prefs.carChosen-1] = 4;
							else if (focusOn == 
								FrmGetObjectIndex(form, jMileageNoteField))
								prefs.whichHasFocus[prefs.carChosen-1] = 5;

							// save fields before leaving this form
							SaveFieldsToPrefs(form);

							prefs.carChosen = selection+1;
							FillCarListArray(); // fill jMRTArrayH

							// simulate frmOpenEvent to the form to reload stuff
							newEvent.eType = frmOpenEvent;
							newEvent.data.frmOpen.formID=jMileageForm;
							EvtAddEventToQueue(&newEvent);
						}
					}
					// if edit categories item
					else if (selection >= 3)
					{
						// open categories form to edit cars
						FrmPopupForm(jMileageEditCategoriesForm);
						handled = true; // set handled true here; don't want
						                // Edit Categories to be the label
					}
					// Don't set handled = true except in specific cases
					break;
				}
			}
			break;
		}

		// keyDownEvent: when a key pressed
		case keyDownEvent:
		{
			UInt16 focusID = FrmGetFocus(form);

			switch (event->data.keyDown.chr)
			{
				// prevFieldChr: move to the previous field
				case prevFieldChr:
					if (focusID==FrmGetObjectIndex(form, jMileageStartField))
						FrmSetFocus(form, 
							FrmGetObjectIndex(form, jMileageNoteField));
					else if (focusID==FrmGetObjectIndex(form, 
					jMileageEndField))
						FrmSetFocus(form, 
							FrmGetObjectIndex(form, jMileageStartField));
					else if (focusID==FrmGetObjectIndex(form, 
					jMileageRefillField))
						FrmSetFocus(form, 
							FrmGetObjectIndex(form, jMileageEndField));
					else if (focusID==FrmGetObjectIndex(form, 
					jMileageCPUField))
						FrmSetFocus(form, 
							FrmGetObjectIndex(form, jMileageRefillField));
					else if (focusID==FrmGetObjectIndex(form, 
					jMileageNoteField))
						FrmSetFocus(form, 
							FrmGetObjectIndex(form, jMileageCPUField));
					handled = true;
					break;

				// nextFieldChr: move to the next field
				case nextFieldChr:
					if (focusID==FrmGetObjectIndex(form, jMileageStartField))
						FrmSetFocus(form, 
							FrmGetObjectIndex(form, jMileageEndField));
					else if (focusID==FrmGetObjectIndex(form, jMileageEndField))
						FrmSetFocus(form, 
							FrmGetObjectIndex(form, jMileageRefillField));
					else if (focusID==FrmGetObjectIndex(form, 
					jMileageRefillField))
						FrmSetFocus(form, 
							FrmGetObjectIndex(form, jMileageCPUField));
					else if (focusID==FrmGetObjectIndex(form, 
					jMileageCPUField))
						FrmSetFocus(form, 
							FrmGetObjectIndex(form, jMileageNoteField));
					else if (focusID==FrmGetObjectIndex(form, 
					jMileageNoteField))
						FrmSetFocus(form, 
							FrmGetObjectIndex(form, jMileageStartField));
					handled = true;
					break;

				// pageDownChr: move the selection of the list down by one
				case pageDownChr:
				{
					EventType newEvent;
					ListType *list;
					Int16 lstSel;

					list = FrmGetObjectPtr(form, 
						FrmGetObjectIndex(form, jMileageOldEntryList));
					lstSel = LstGetSelection(list);
					if (lstSel==noListSelection)
						lstSel = jMRTArrayNum-1;
					else
					{
						lstSel = lstSel + 1;
						if (lstSel == jMRTArrayNum) lstSel = 0;
					}
					// Simulate a selection on the entry lstSel
					LstSetSelection(list, lstSel);
					LstDrawList(list);
					newEvent.eType = lstSelectEvent;
					newEvent.data.lstSelect.listID=jMileageOldEntryList;
					newEvent.data.lstSelect.pList = list;
					newEvent.data.lstSelect.selection = lstSel;
					EvtAddEventToQueue(&newEvent);

					handled = true;
					break;
				}

				case pageUpChr:
				{
					EventType newEvent;
					ListType *list;
					Int16 lstSel;

					list = FrmGetObjectPtr(form, 
						FrmGetObjectIndex(form, jMileageOldEntryList));
					lstSel = LstGetSelection(list);
					if (lstSel==noListSelection)
						lstSel = jMRTArrayNum-1;
					else
					{
						lstSel = lstSel - 1;
						if (lstSel < 0) lstSel = jMRTArrayNum - 1;
					}
					// Simulate a selection on the entry lstSel
					LstSetSelection(list, lstSel);
					LstDrawList(list);
					newEvent.eType = lstSelectEvent;
					newEvent.data.lstSelect.listID=jMileageOldEntryList;
					newEvent.data.lstSelect.pList = list;
					newEvent.data.lstSelect.selection = lstSel;
					EvtAddEventToQueue(&newEvent);

					handled = true;
					break;
				}
				
				default:
					break;
			}
			break;
		}


		// menuEvent: when a menu is chosen
		case menuEvent:
		{
			switch (event->data.menu.itemID)
			{
				case AboutJMileage:
					FrmAlert(AboutAlert);
					handled = true;
					break;

				case MainDetails:
					FrmAlert(DetailsRouteAlert);
					FrmPopupForm(jMileageDetailsForm);
					handled = true;
					break;

				case MainEdit:
				{
					// bring up the edit screen if any item is selected
					ListType *list = FrmGetObjectPtr(form,
						FrmGetObjectIndex(form, jMileageOldEntryList));
					listSel = LstGetSelection(list);
					if (listSel == noListSelection)
					{
						// no item was selected to delete - ALERT
						FrmAlert(EditAlert);
					}
					else
					{
						// item selected - popup form
						FrmPopupForm(jMileageEditForm);
					}
					break;
				}

				case MainDelete:
				{
					// delete the item selected from jMileage's list
					ListType *list = FrmGetObjectPtr(form, 
						FrmGetObjectIndex(form, jMileageOldEntryList));
					Int16 listSel = LstGetSelection(list);
					if (listSel == noListSelection)
					{
						// no item was selected to delete - ALERT
						FrmAlert(DeleteAlert);
					}
					else
					{
						jMileageRecordType *jMRTArray;
						jMileageRecordType *jMRT1, *jMRT2;
						Int16 i, index;
						Boolean found = false;
						MemHandle mh;
						EventType newEvent;
						Boolean delete = false;

						switch(FrmAlert(DeleteConfAlert))
						{
							case 0: // Yes
								delete = true;
								break;
							case 1: // No
								delete = false;
								break;
						}

						if (delete)
						{
							jMRTArray = MemHandleLock(jMRTArrayH);
							jMRT1 = jMRTArray + listSel; // elmt to be deleted

							// delete record from the database
							//  first find the location of the record to delete
							index = FindIndexInDB(jMRT1);
							MemHandleUnlock(jMRTArrayH);

							if (index > -1)
							{
								// delete selected item
								DeleteFromDB(index);

								FillCarListArray();

								// draw average MPG
								DrawAMPG();
								ClearMPG();

								// set the form's list size to jMRTArrayNum
								LstSetListChoices(list, NULL, jMRTArrayNum);
								LstSetSelection(list, -1); // clear any sel
								LstDrawList(list);
							}
						}
					}
					handled = true;
					break;
				}
				case MainPreferences:
				{
					FrmPopupForm(jMileagePreferencesForm);
					handled = true;
					break;
				}
				case DataExport:
				{
					ExportData();
					break;
				}

				default: 
					break;
			}
			break;
		}

		// penUpEvent: if in bottom third, bring up details screen
		case penDownEvent:
		{
			if (event->screenY > 114)
			{
				FrmPopupForm(jMileageDetailsForm);
				handled = true;
			}
			break;
		}

		default:
			break;
	}

#ifdef __GNUC__
CALLBACK_EPILOGUE;
#endif

	return handled;
}








// ***** EditCategoriesFormEventHandler *****
// Handle events for jMileageEditCategoriesForm form
Boolean EditCategoriesFormEventHandler(EventPtr event)
{
	Boolean handled = false;
	FormType *form;

#ifdef __GNUC__
CALLBACK_PROLOGUE;
#endif

	form = FrmGetActiveForm();

	switch (event->eType)
	{
		// frmOpenEvent: the main form has been opened
		case frmOpenEvent:
		{
			FieldType *field;
			MemHandle mh1, mh2;
			Char *str;

			// set prefs.car1 into EditCategories1Field
			field = FrmGetObjectPtr(form, 
				FrmGetObjectIndex(form, EditCategories1Field));
			mh1 = MemHandleNew(21);
			str = MemHandleLock(mh1);
			StrCopy(str, prefs.car1);
			MemHandleUnlock(mh1);
			mh2 = FldGetTextHandle(field);
			FldSetTextHandle(field, mh1);
			FldDrawField(field);
			if (mh2) MemHandleFree(mh2);

			// select EditCategories1Field text and set focus on it
			FldSetSelection(field, 0, StrLen(prefs.car1)*sizeof(Char));
			FrmSetFocus(form, FrmGetObjectIndex(form, EditCategories1Field));

			// set prefs.car2 into EditCategories2Field
			field = FrmGetObjectPtr(form, 
				FrmGetObjectIndex(form, EditCategories2Field));
			mh1 = MemHandleNew(21);
			str = MemHandleLock(mh1);
			StrCopy(str, prefs.car2);
			MemHandleUnlock(mh1);
			mh2 = FldGetTextHandle(field);
			FldSetTextHandle(field, mh1);
			FldDrawField(field);
			if (mh2) MemHandleFree(mh2);

			// set prefs.car3 into EditCategories3Field
			field = FrmGetObjectPtr(form, 
				FrmGetObjectIndex(form, EditCategories3Field));
			mh1 = MemHandleNew(21);
			str = MemHandleLock(mh1);
			StrCopy(str, prefs.car3);
			MemHandleUnlock(mh1);
			mh2 = FldGetTextHandle(field);
			FldSetTextHandle(field, mh1);
			FldDrawField(field);
			if (mh2) MemHandleFree(mh2);

			FrmDrawForm(form);
			handled = true;
			break;
		}

		// ctlSelectEvent: OK or Cancel
		case ctlSelectEvent:
		{
			switch (event->data.ctlSelect.controlID)
			{
				case EditCategoriesOKBtn:
				{
					FormType *origForm = FrmGetFormPtr(jMileageForm);
					ListType *oCarList = FrmGetObjectPtr(origForm,
						FrmGetObjectIndex(origForm, jMileageCarList));
					Char *str;
					FieldType *field;
					
					// Copy into car1
					field = FrmGetObjectPtr(form,
						FrmGetObjectIndex(form, EditCategories1Field));
					str = FldGetTextPtr(field);
					if (str!=NULL)
						StrCopy(prefs.car1, str);
					else
						StrCopy(prefs.car1, " ");

					// Copy into car2
					field = FrmGetObjectPtr(form,
						FrmGetObjectIndex(form, EditCategories2Field));
					str = FldGetTextPtr(field);
					if (str!=NULL)
						StrCopy(prefs.car2, str);
					else
						StrCopy(prefs.car2, " ");

					// Copy into car3
					field = FrmGetObjectPtr(form,
						FrmGetObjectIndex(form, EditCategories3Field));
					str = FldGetTextPtr(field);
					if (str!=NULL)
						StrCopy(prefs.car3, str);
					else
						StrCopy(prefs.car3, " ");

					LstSetSelection(oCarList, prefs.carChosen-1);
					FrmReturnToForm(0);
					FrmUpdateForm(jMileageForm, frmRedrawUpdateCode);
					handled = true;
					break;
				}

				case EditCategoriesCancelBtn:
				{
					FormType *origForm = FrmGetFormPtr(jMileageForm);
					ListType *oCarList = FrmGetObjectPtr(origForm,
						FrmGetObjectIndex(origForm, jMileageCarList));
					LstSetSelection(oCarList, prefs.carChosen-1);
					FrmReturnToForm(0);
					FrmUpdateForm(jMileageForm, frmRedrawUpdateCode);
					handled = true;
					break;
				}
			}
			break;
		}

		// keyDownEvent: when a key pressed
		case keyDownEvent:
		{
			UInt16 focusID = FrmGetFocus(form);

			switch (event->data.keyDown.chr)
			{
				case prevFieldChr:
					if (focusID==FrmGetObjectIndex(form, EditCategories1Field))
						FrmSetFocus(form, 
							FrmGetObjectIndex(form, EditCategories3Field));
					else if (focusID==FrmGetObjectIndex(form, 
					EditCategories2Field))
						FrmSetFocus(form, 
							FrmGetObjectIndex(form, EditCategories1Field));
					else if (focusID==FrmGetObjectIndex(form, 
					EditCategories3Field))
						FrmSetFocus(form, 
							FrmGetObjectIndex(form, EditCategories2Field));
					handled = true;
					break;

				case nextFieldChr:
					if (focusID==FrmGetObjectIndex(form, EditCategories1Field))
						FrmSetFocus(form, 
							FrmGetObjectIndex(form, EditCategories2Field));
					else if (focusID==FrmGetObjectIndex(form, 
					EditCategories2Field))
						FrmSetFocus(form, 
							FrmGetObjectIndex(form, EditCategories3Field));
					else if (focusID==FrmGetObjectIndex(form, 
					EditCategories3Field))
						FrmSetFocus(form, 
							FrmGetObjectIndex(form, EditCategories1Field));
					handled = true;
					break;
				
				default:
					break;
			}
			break;
		}

		default:
			break;
	}

#ifdef __GNUC__
CALLBACK_EPILOGUE;
#endif

	return handled;
}
	








// ***** PreferencesFormEventHandler *****
// Event handler for PreferencesForm
Boolean PreferencesFormEventHandler(EventPtr event)
{

	Boolean handled = false;
	FormType *form;

#ifdef __GNUC__
CALLBACK_PROLOGUE;
#endif

	form = FrmGetActiveForm();

	switch (event->eType)
	{
		// frmOpenEvent: the preferences form has been opened
		case frmOpenEvent:
		{
			ControlType *ctl = NULL;
			ListType *list;
			Char *label;

			// set Label pushbuttons (group 1)
			if (prefs.prefLabelType == MPG)
			{
				ctl = FrmGetObjectPtr(form, 
					FrmGetObjectIndex(form, PrefMPGPush));
				CtlSetValue(ctl, 1);
			}
			else if (prefs.prefLabelType == LP100KM)
			{
				ctl = FrmGetObjectPtr(form, 
					FrmGetObjectIndex(form, PrefKPLPush));
				CtlSetValue(ctl, 1);
			}
			else if (prefs.prefLabelType == KMPL)
			{
				ctl = FrmGetObjectPtr(form, 
					FrmGetObjectIndex(form, PrefKmPLPush));
				CtlSetValue(ctl, 1);
			}
			if (ctl) CtlDrawControl(ctl);
				
			// set Date pushbuttons (group 1)
			ctl = NULL;
			if (prefs.prefDateType == MSDSY)
			{
				ctl = FrmGetObjectPtr(form,
					FrmGetObjectIndex(form, PrefMDYsPush));
				CtlSetValue(ctl, 1);
			}
			else if (prefs.prefDateType == DPMPY)
			{
				ctl = FrmGetObjectPtr(form,
					FrmGetObjectIndex(form, PrefDMYpPush));
				CtlSetValue(ctl, 1);
			}
			else if (prefs.prefDateType == DSMSY)
			{
				ctl = FrmGetObjectPtr(form,
					FrmGetObjectIndex(form, PrefDMYsPush));
				CtlSetValue(ctl, 1);
			}
			else if (prefs.prefDateType == DDMDY)
			{
				ctl = FrmGetObjectPtr(form,
					FrmGetObjectIndex(form, PrefDMYdPush));
				CtlSetValue(ctl, 1);
			}
			if (ctl) CtlDrawControl(ctl);
				
			FrmDrawForm(form);
			handled = true;
			break;
		}
		
		// ctlSelectEvent: a button was chosen (or some other control select)
		case ctlSelectEvent:
		{
			switch (event->data.ctlSelect.controlID)
			{
				case PrefOKBtn:
				{
					FormType *origForm = FrmGetFormPtr(jMileageForm);
					ListType *oCarList = FrmGetObjectPtr(origForm,
						FrmGetObjectIndex(origForm, jMileageCarList));
					ControlType *ctl;
					ListType *list;
					UInt16 oldLabelType;
					UInt16 objIndex;

					// store values into prefs
					oldLabelType = prefs.prefLabelType;
					objIndex = FrmGetControlGroupSelection(form, 1);
					if (objIndex == FrmGetObjectIndex(form, PrefMPGPush))
						prefs.prefLabelType = MPG;
					else if (objIndex == FrmGetObjectIndex(form, PrefKPLPush))
						prefs.prefLabelType = LP100KM;
					else if (objIndex == FrmGetObjectIndex(form, PrefKmPLPush))
						prefs.prefLabelType = KMPL;
					objIndex = FrmGetControlGroupSelection(form, 2);
					if (objIndex == FrmGetObjectIndex(form, PrefMDYsPush))
						prefs.prefDateType = MSDSY;
					else if (objIndex == FrmGetObjectIndex(form, PrefDMYpPush))
						prefs.prefDateType = DPMPY;
					else if (objIndex == FrmGetObjectIndex(form, PrefDMYsPush))
						prefs.prefDateType = DSMSY;
					else if (objIndex == FrmGetObjectIndex(form, PrefDMYdPush))
						prefs.prefDateType = DDMDY;
					if (prefs.prefLabelType != oldLabelType)
					{
						// label type changed; reload label
						InitLabels();
					}

					// cleanup
					LstSetSelection(oCarList, prefs.carChosen-1);
					FrmReturnToForm(0);
					FrmUpdateForm(jMileageForm, frmRedrawUpdateCode);
					handled = true;
					break;
				}

				case PrefCancelBtn:
				{
					FormType *origForm = FrmGetFormPtr(jMileageForm);
					ListType *oCarList = FrmGetObjectPtr(origForm,
						FrmGetObjectIndex(origForm, jMileageCarList));
					// cleanup
					LstSetSelection(oCarList, prefs.carChosen-1);
					FrmReturnToForm(0);
					FrmUpdateForm(jMileageForm, frmRedrawUpdateCode);
					handled = true;
					break;
				}
			}
			break;
		}
	}

#ifdef __GNUC__
CALLBACK_EPILOGUE;
#endif

	return handled;
}







// ***** EditFormEventHandler *****
// Event handler for EditForm
Boolean EditFormEventHandler(EventPtr event)
{

	Boolean handled = false;
	FormType *form;

#ifdef __GNUC__
CALLBACK_PROLOGUE;
#endif

	form = FrmGetActiveForm();

	switch (event->eType)
	{
		// frmOpenEvent: the preferences form has been opened
		case frmOpenEvent:
		{
			ControlType *ctl = NULL;
			jMileageRecordType *jMRTArray, *jMRT;
			DateTimeType date;
			FieldType *field;
			MemHandle memH, memH2;
			Char *label, *str;

			// Find entry corresponding to listSel
			jMRTArray = MemHandleLock(jMRTArrayH);
			jMRT = jMRTArray + listSel; // elmt to be edited

			// convert time in record to DateTimeType
			TimSecondsToDateTime(jMRT->date, &date);
			year = date.year;
			month = date.month;
			day = date.day;

			// Set time label
			ctl = FrmGetObjectPtr(form, FrmGetObjectIndex(form, EditSelector));
			label = CtlGetLabel(ctl);
			DateToDOWDMFormat(month, day, year, dateFormatType, label);
			CtlSetLabel(ctl, label);

			// set EditStartField
			memH = MemHandleNew(13);
			str = MemHandleLock(memH);
			if (jMRT->startF >= 0)
				PrintF(str, jMRT->startF, 12, 1);
			else
				MemSet(str, MemHandleSize(memH), 0);
			MemHandleUnlock(memH);
			field = FrmGetObjectPtr(form, 
				FrmGetObjectIndex(form, EditStartField));
			memH2 = FldGetTextHandle(field);
			FldSetTextHandle(field, memH);
			FldDrawField(field);
			if (memH2) MemHandleFree(memH2);

			// set EditEndField
			memH = MemHandleNew(13);
			str = MemHandleLock(memH);
			if (jMRT->endF >= 0)
				PrintF(str, jMRT->endF, 12, 1);
			else
				MemSet(str, MemHandleSize(memH), 0);
			MemHandleUnlock(memH);
			field = FrmGetObjectPtr(form, 
				FrmGetObjectIndex(form, EditEndField));
			memH2 = FldGetTextHandle(field);
			FldSetTextHandle(field, memH);
			FldDrawField(field);
			if (memH2) MemHandleFree(memH2);

			// set EditRefillField
			memH = MemHandleNew(13);
			str = MemHandleLock(memH);
			if (jMRT->refillF >= 0)
				PrintF(str, jMRT->refillF, 12, 3);
			else
				MemSet(str, MemHandleSize(memH), 0);
			MemHandleUnlock(memH);
			field = FrmGetObjectPtr(form, 
				FrmGetObjectIndex(form, EditRefillField));
			memH2 = FldGetTextHandle(field);
			FldSetTextHandle(field, memH);
			FldDrawField(field);
			if (memH2) MemHandleFree(memH2);

			// set EditCPUField
			memH = MemHandleNew(7);
			str = MemHandleLock(memH);
			if (jMRT->cpuF >= 0)
				PrintF(str, jMRT->cpuF, 5, 3);
			else
				MemSet(str, MemHandleSize(memH), 0);
			MemHandleUnlock(memH);
			field = FrmGetObjectPtr(form, 
				FrmGetObjectIndex(form, EditCPUField));
			memH2 = FldGetTextHandle(field);
			FldSetTextHandle(field, memH);
			FldDrawField(field);
			if (memH2) MemHandleFree(memH2);

			// set EditNoteField
			memH = MemHandleNew(31);
			str = MemHandleLock(memH);
			MemSet(str, MemHandleSize(memH), 0);
			StrCopy(str, jMRT->notes);
			MemHandleUnlock(memH);
			field = FrmGetObjectPtr(form, 
				FrmGetObjectIndex(form, EditNoteField));
			memH2 = FldGetTextHandle(field);
			FldSetTextHandle(field, memH);
			FldDrawField(field);
			if (memH2) MemHandleFree(memH2);

			MemHandleUnlock(jMRTArrayH);

			// standard frmOpenEvent closer
			FrmSetFocus(form, FrmGetObjectIndex(form, EditStartField));
			FrmDrawForm(form);
			handled = true;
			break;
		}

		// ctlSelectEvent: OK or Cancel
		case ctlSelectEvent:
		{
			switch (event->data.ctlSelect.controlID)
			{
				// EditSelector: show a date picker dialog
				case EditSelector:
				{
					ControlType *ctl;
					Char *label;
					Int16 index;
					DateTimeType date;
					jMileageRecordType *jMRTArray, *jMRT;

					// Get record of selected item
					jMRTArray = MemHandleLock(jMRTArrayH);
					jMRT = jMRTArray + listSel; // elmt to be edited

					// convert time in record to DateTimeType
					TimSecondsToDateTime(jMRT->date, &date);
					// set year, month, day from date.year, ...
					year = date.year;
					month = date.month;
					day = date.day;

					// do SelectDay (see p. 231)
					if (SelectDay(selectDayByDay, &month, &day, &year,
						"Select a Day"))
					{
						ctl = FrmGetObjectPtr(form,
							FrmGetObjectIndex(form, EditSelector));
						label = CtlGetLabel(ctl);
						DateToDOWDMFormat(month, day, year, dateFormatType,
							label);
						CtlSetLabel(ctl, label);
					}
					MemHandleUnlock(jMRTArrayH);

					handled = true;
					break;
				}
				// EditOKBtn: save data if possible
				case EditOKBtn:
				{
					FormType *origForm = FrmGetFormPtr(jMileageForm);
					ListType *oCarList = FrmGetObjectPtr(origForm,
						FrmGetObjectIndex(origForm, jMileageCarList));
					FieldType *field;
					ControlType *ctl;
					DateTimeType dt;
					Char *str;
					jMileageRecordType *jMRT, *jMRTArray;
					UInt8 error = 0;
					float startF, endF, refillF, cpuF;
					Char note[31];
					UInt32 date;
					Int16 indexNum, newIndexNum;

					// Find entry corresponding to listSel
					jMRTArray = MemHandleLock(jMRTArrayH);
					jMRT = jMRTArray + listSel; // elmt to be edited

					// get and validate startF
					field = FrmGetObjectPtr(form, 
						FrmGetObjectIndex(form, EditStartField));
					str = FldGetTextPtr(field);
					if (str==NULL || StrLen(str)==0)
						error = 1;
					else
					{
						// Parse start mileage into f
						float f = GccSysFlpAToF(str);
						if (f < 0)
							error = 1; // invalid f
						else
							startF = f;
					}
					
					// get and validate endF
					field = FrmGetObjectPtr(form, 
						FrmGetObjectIndex(form, EditEndField));
					str = FldGetTextPtr(field);
					if (str==NULL || StrLen(str)==0)
						error = 2;
					else
					{
						// Parse end mileage into f
						float f = GccSysFlpAToF(str);
						if (f < 0)
							error = 2; // invalid f
						else
							endF = f;
					}

					// get and validate refillF
					field = FrmGetObjectPtr(form, 
						FrmGetObjectIndex(form, EditRefillField));
					str = FldGetTextPtr(field);
					if (str==NULL || StrLen(str)==0)
						error = 3;
					else
					{
						// Parse refill mileage into f
						float f = GccSysFlpAToF(str);
						if (f <= 0)
							error = 3; // invalid f
						else
							refillF = f;
					}

					// get and validate cpuF
					field = FrmGetObjectPtr(form, 
						FrmGetObjectIndex(form, EditCPUField));
					str = FldGetTextPtr(field);
					if (str==NULL || StrLen(str)==0)
						cpuF = -1;
					else
					{
						// Parse cost per unit into f
						float f = GccSysFlpAToF(str);
						if (f <= 0)
							error = 4; // invalid f
						else
							cpuF = f;
					}

					// get note
					field = FrmGetObjectPtr(form, 
						FrmGetObjectIndex(form, EditNoteField));
					str = FldGetTextPtr(field);
					StrCopy(note, str);

					// get date
					MemSet(&dt, sizeof(dt), 0);
					dt.year = year;
					dt.month = month;
					dt.day = day;
					date = TimDateTimeToSeconds(&dt);
					
					// find entry first
					indexNum = FindIndexInDB(jMRT);
					if (indexNum==-1)
						error = 10;
					else
					{
						// entry found at entryNum; change it!
						MemHandle recordH;
						jMileageRecordType *record;
						jMileageRecordType *cmp;
						UInt16 attributes;
						Boolean move = true;
						Int16 newIndexNum;

						// set items
						jMRT->startF = startF;
						jMRT->endF = endF;
						jMRT->refillF = refillF;
						jMRT->cpuF = cpuF;
						StrCopy(jMRT->notes, note);
						jMRT->date = date;
						
						//XXX Make this better!  Ugh, very hard to!
						// delete
						DeleteFromDB(indexNum);
						// add
						AddToDB(jMRT);
					}

					// insertion sort to resort jMRTArrayH data
					SysInsertionSort(jMRTArray, jMRTArrayNum, 
						sizeof(jMileageRecordType), 
						(const CmpFuncPtr) &(jMRCompareFunc), 0);

					MemHandleUnlock(jMRTArrayH);


					// cleanup
					if (!error)
					{
						LstSetSelection(oCarList, prefs.carChosen-1);
						LstDrawList(oCarList);
						FrmReturnToForm(0);
						FrmUpdateForm(jMileageForm, frmRedrawUpdateCode);
					}
					handled = true;
					break;
				}
				case EditCancelBtn:
				{
					FormType *origForm = FrmGetFormPtr(jMileageForm);
					ListType *oCarList = FrmGetObjectPtr(origForm,
						FrmGetObjectIndex(origForm, jMileageCarList));
					// cleanup
					LstSetSelection(oCarList, prefs.carChosen-1);
					FrmReturnToForm(0);
					FrmUpdateForm(jMileageForm, frmRedrawUpdateCode);
					handled = true;
					break;
				}
			}
			break;
		}
	}

#ifdef __GNUC__
CALLBACK_EPILOGUE;
#endif

	return handled;
}





// ***** DetailsFormEventHandler *****
// Event handler for DetailsForm
Boolean DetailsFormEventHandler(EventPtr event)
{

	Boolean handled = false;
	FormType *form;

#ifdef __GNUC__
CALLBACK_PROLOGUE;
#endif

	form = FrmGetActiveForm();

	switch (event->eType)
	{
		// frmOpenEvent: the preferences form has been opened
		case frmOpenEvent:
		{
			MemHandle memH, memH2;
			Char *str;
			FormType *oldForm;
			FieldType *field;
			ListType *list;
			Int16 selection;
			jMileageRecordType *jMRTArray, *jMRT;
			float TCost, TCPM;
			float ACPM, ACPTDay, ACPTMonth, ACPTYear;
			UInt32 ANumDays, ANumMonths, ANumYears;
			FlpCompFloat flpfloat;
			UInt16 i;

			jMRTArray = MemHandleLock(jMRTArrayH);

			// Get the jMileageOldEntryList pointer
			oldForm = FrmGetFormPtr(jMileageForm);
			list = FrmGetObjectPtr(oldForm, 
				FrmGetObjectIndex(oldForm, jMileageOldEntryList));
			// Get the selection of the list
			selection = LstGetSelection(list);
			// if an item is selected in the list
			if (selection >= 0)
			{
				// get the jMileageRecordType entry related to it
				jMRT = jMRTArray + selection;

				// display the total cost or a blank in DetailsTCost
				TCost = jMRT->cpuF * jMRT->refillF;
				memH = MemHandleNew(9);
				str = MemHandleLock(memH);
				if (TCost >= 0)
					PrintF(str, TCost, 8, 2);
				else
					MemSet(str, MemHandleSize(memH), 0);
				MemHandleUnlock(memH);
				field = FrmGetObjectPtr(form, 
					FrmGetObjectIndex(form, DetailsTCost));
				memH2 = FldGetTextHandle(field);
				FldSetTextHandle(field, memH);
				FldDrawField(field);
				if (memH2) MemHandleFree(memH2);

				// display the cost per unit distance or a blank in DetailsTCPM
				TCPM = TCost / (jMRT->endF - jMRT->startF);
				memH = MemHandleNew(8);
				str = MemHandleLock(memH);
				if (TCPM >= 0)
					PrintF(str, TCPM, 7, 4);
				else
					MemSet(str, MemHandleSize(memH), 0);
				MemHandleUnlock(memH);
				field = FrmGetObjectPtr(form, 
					FrmGetObjectIndex(form, DetailsTCPM));
				memH2 = FldGetTextHandle(field);
				FldSetTextHandle(field, memH);
				FldDrawField(field);
				if (memH2) MemHandleFree(memH2);
			}
			// Calculate averages
			ACPM = 0;
			ACPTDay = 0;
			ANumDays = 1;
			for (i = 0; i < jMRTArrayNum; i++)
			{
				jMRT = jMRTArray + i;
				if (jMRT->cpuF > 0 && jMRT->refillF > 0)
				{
					TCost = jMRT->cpuF * jMRT->refillF;
					TCPM = TCost / (jMRT->endF - jMRT->startF);
					ACPM = ACPM + TCPM;
				}
				if (jMRT->cpuF > 0 && i != 0)
				{
					jMileageRecordType *jMRTOld = jMRTArray + i - 1;
					DateType date1, date2;
					UInt32 temp1, temp2;
					DateSecondsToDate(jMRT->date, &date1);
					DateSecondsToDate(jMRTOld->date, &date2);
					ACPTDay += jMRT->cpuF * jMRT->refillF;
					temp1 = DateToDays(date1);
					temp2 = DateToDays(date2);
					ANumDays += DateToDays(date1)-DateToDays(date2);
				}
			}
			flpfloat.ff = _f_itof(jMRTArrayNum);
			ACPM = ACPM / flpfloat.f;
			flpfloat.ff = _f_itof(ANumDays);
			ACPTDay = ACPTDay / flpfloat.f;

			// Fill Avg CPM
			memH = MemHandleNew(8);
			str = MemHandleLock(memH);
			if (ACPM >= 0)
				PrintF(str, ACPM, 7, 4);
			else
				MemSet(str, MemHandleSize(memH), 0);
			MemHandleUnlock(memH);
			field = FrmGetObjectPtr(form, 
				FrmGetObjectIndex(form, DetailsACPM));
			memH2 = FldGetTextHandle(field);
			FldSetTextHandle(field, memH);
			FldDrawField(field);
			if (memH2) MemHandleFree(memH2);

			// Fill Avg cost per day
			memH = MemHandleNew(8);
			str = MemHandleLock(memH);
			if (ACPTDay >= 0)
				PrintF(str, ACPTDay, 7, 4);
			else
				MemSet(str, MemHandleSize(memH), 0);
			MemHandleUnlock(memH);
			field = FrmGetObjectPtr(form, 
				FrmGetObjectIndex(form, DetailsACPTDay));
			memH2 = FldGetTextHandle(field);
			FldSetTextHandle(field, memH);
			FldDrawField(field);
			if (memH2) MemHandleFree(memH2);
			
			MemHandleUnlock(jMRTArrayH);

			// standard frmOpenEvent closer
			FrmDrawForm(form);
			handled = true;
			break;
		}

		// ctlSelectEvent: Close
		case ctlSelectEvent:
		{
			if (event->data.ctlSelect.controlID==DetailsCloseBtn)
			{
				FormType *origForm = FrmGetFormPtr(jMileageForm);
				ListType *oCarList = FrmGetObjectPtr(origForm,
					FrmGetObjectIndex(origForm, jMileageCarList));
				// cleanup
				LstSetSelection(oCarList, prefs.carChosen-1);
				FrmReturnToForm(0);
				FrmUpdateForm(jMileageForm, frmRedrawUpdateCode);
				handled = true;
				break;
			}
			break;
		}
	}

#ifdef __GNUC__
CALLBACK_EPILOGUE;
#endif

	return handled;
}








// ***** DrawAMPG *****
static void DrawAMPG()
{
	jMileageRecordType *jMRTArray;
	float milesSum, refillSum, avg;
	RectangleType rect;
	Char *str;
	Char tmpStr[25];
	FontID oldFont;
	UInt16 i;

	// clear rectangle
	rect.topLeft.x = 0;
	rect.topLeft.y = 145;
	rect.extent.x = 80;
	rect.extent.y = 15;
	WinEraseRectangle(&rect, 0);

	if (jMRTArrayNum==0) 
		return;

	// calculate avg
	milesSum = refillSum = 0;
	jMRTArray = MemHandleLock(jMRTArrayH);
	for (i = 0; i < jMRTArrayNum; i++)
	{
		jMileageRecordType *jMRT = jMRTArray+i;
		milesSum = milesSum + (jMRT->endF - jMRT->startF);
		refillSum = refillSum + jMRT->refillF;
	}
	if (refillSum==0) return;
	avg = milesSum / refillSum;
	if (prefs.prefLabelType == LP100KM)
		avg = 1/avg * 100; // change for L/100Km
	PrintF(tmpStr, avg, 6, 3);
	MemHandleUnlock(jMRTArrayH);

	// draw Label
	rect.topLeft.x = 3;
	rect.topLeft.y = 145;
	str = MemHandleLock(AMPGStringH);
	WinDrawChars(str, StrLen(str), 
		rect.topLeft.x, rect.topLeft.y);
	MemHandleUnlock(AMPGStringH);

	// draw Data
	oldFont = FntSetFont(boldFont);
	rect.topLeft.x = 23;
	rect.topLeft.y = 145;
	WinDrawChars(tmpStr, StrLen(tmpStr),
		rect.topLeft.x, rect.topLeft.y);
	FntSetFont(oldFont);
}






// ***** DrawMPG *****
static void DrawMPG(UInt8 selection)
{
	RectangleType *rect;
	UInt16 x, y;
	Char *str;
	FontID oldFont;
	Char tmpStr[21];
	UInt8 sizeof_tmpStr = 21;
	float resultF;
	jMileageRecordType *jMRTArray, *jMRT;

	// get entry
	jMRTArray = MemHandleLock(jMRTArrayH);
	jMRT = jMRTArray + selection;

	// calculate resultF
	resultF = (jMRT->endF - jMRT->startF) 
		/ jMRT->refillF;
	if (prefs.prefLabelType == LP100KM)
		resultF = 1/resultF * 100; // change for L/100Km
	MemSet(tmpStr, sizeof_tmpStr, 0);
	PrintF(tmpStr, resultF, 6, 3);

	// draw MPG Label
	if (prefs.prefLabelType==MPG)
		x = 80;
	else if (prefs.prefLabelType==LP100KM)
		x = 70;
	else if (prefs.prefLabelType==KMPL)
		x = 80;
	y = 145;
	str = MemHandleLock(MPGStringH);
	WinDrawChars(str, StrLen(str), x, y);
	MemHandleUnlock(MPGStringH);

	// draw MPG Data
	oldFont = FntSetFont(boldFont);
	if (prefs.prefLabelType==MPG)
		x = 105;
	else if (prefs.prefLabelType==LP100KM)
		x = 115;
	else if (prefs.prefLabelType==KMPL)
		x = 110;
	y = 145;
	WinDrawChars(tmpStr, StrLen(tmpStr), x, y);
	FntSetFont(oldFont);

	// draw Refill Label
	x = 3;
	y = 130;
	MemSet(tmpStr, sizeof_tmpStr, 0);
	StrCopy(tmpStr, "Refill:");
	WinDrawChars(tmpStr, StrLen(tmpStr), x, y);

	// draw Refill Data
	x = 28;
	MemSet(tmpStr, sizeof_tmpStr, 0);
	PrintF(tmpStr, jMRT->refillF, 12, 3);
	WinDrawChars(tmpStr, StrLen(tmpStr), x, y);

	if (jMRT->cpuF > 0)
	{
		// draw CPU Label
		x = 80;
		y = 130;
		MemSet(tmpStr, sizeof_tmpStr, 0);
		StrCopy(tmpStr, "C/u qty:");
		WinDrawChars(tmpStr, StrLen(tmpStr), x, y);

		// draw Refill Data
		x = 117;
		MemSet(tmpStr, sizeof_tmpStr, 0);
		PrintF(tmpStr, jMRT->cpuF, 12, 3);
		WinDrawChars(tmpStr, StrLen(tmpStr), x, y);
	}

	if (jMRT->notes[0] != 0)
	{
		// draw Note Label
		x = 3;
		y = 115;
		MemSet(tmpStr, sizeof_tmpStr, 0);
		StrCopy(tmpStr, "Note:");
		WinDrawChars(tmpStr, StrLen(tmpStr), x, y);

		// draw Note Data
		x = 28;
		WinDrawChars(jMRT->notes, StrLen(jMRT->notes), x, y);
	}

}





// ***** ClearAllMPG *****
// Clears the entire bottom section, below the list
static void ClearAllMPG()
{
	RectangleType rect;

	// erase output area
	rect.topLeft.x = 0;
	rect.topLeft.y = 115;
	rect.extent.x = 160;
	rect.extent.y = 160-115;
	WinEraseRectangle(&rect, 0);
}







// ***** ClearMPG *****
static void ClearMPG()
{
	RectangleType rect;

	// erase output area
	rect.topLeft.x = 70;
	rect.topLeft.y = 145;
	rect.extent.x = 90;
	rect.extent.y = 15;
	WinEraseRectangle(&rect, 0);
	rect.topLeft.x = 0;
	rect.topLeft.y = 115;
	rect.extent.x = 160;
	rect.extent.y = 30;
	WinEraseRectangle(&rect, 0);
}





// ***** ListDrawFn *****
// List drawing callback function for jMileageOldEntryList
static void ListDrawFn(UInt16 itemNum, RectangleType *bounds,
	Char **itemsText)
{
	jMileageRecordType *jMRTArray;
	jMileageRecordType *jMRT;
	Char tmpStr[30];
	UInt8 strPos;
	DateTimeType dateTime;
	FlpCompFloat iValueFlpF;
	Int32 iValueD;

#ifdef __GNUC__
CALLBACK_PROLOGUE;
#endif

	jMRTArray = MemHandleLock(jMRTArrayH);
	jMRT = jMRTArray+itemNum;

	// determine DateFormatType dft
	if (prefs.prefDateType == MSDSY)
		dateFormatType = dfMDYWithSlashes;
	else if (prefs.prefDateType == DPMPY)
		dateFormatType = dfDMYWithDots;
	else if (prefs.prefDateType == DSMSY)
		dateFormatType = dfDMYWithSlashes;
	else if (prefs.prefDateType == DDMDY)
		dateFormatType = dfDMYWithDashes;

	// draw date
	MemSet(tmpStr, sizeof(tmpStr)/sizeof(Char), 0);
	TimSecondsToDateTime(jMRT->date, &dateTime);
	DateToAscii(dateTime.month, dateTime.day, dateTime.year, dateFormatType, 
		tmpStr);
	tmpStr[StrLen(tmpStr)] = ':';
	WinDrawChars(tmpStr, StrLen(tmpStr),
		bounds->topLeft.x + 0, bounds->topLeft.y);

	// draw start mileage
	MemSet(tmpStr, sizeof(tmpStr)/sizeof(Char), 0);
	//PrintF(tmpStr, jMRT->startF, 12, 0);
	iValueFlpF.f = jMRT->startF;
	iValueD = _f_ftou(iValueFlpF.ff);
	StrPrintF(tmpStr, "%ld", iValueD);

	WinDrawChars(tmpStr, StrLen(tmpStr),
		bounds->topLeft.x + 44, bounds->topLeft.y);
	
	// draw end mileage
	MemSet(tmpStr, sizeof(tmpStr)/sizeof(Char), 0);
	//PrintF(tmpStr, jMRT->endF, 12, 0);
	iValueFlpF.f = jMRT->endF;
	iValueD = _f_ftou(iValueFlpF.ff);
	StrPrintF(tmpStr, "%ld", iValueD);

	WinDrawChars(tmpStr, StrLen(tmpStr),
		bounds->topLeft.x + 100, bounds->topLeft.y);

	// draw middle divider (-->)
	WinDrawChars("->",StrLen("->"), bounds->topLeft.x+85, bounds->topLeft.y);

	MemHandleUnlock(jMRTArrayH);

#ifdef __GNUC__
CALLBACK_EPILOGUE;
#endif

}









// ***** ApplicationHandleEvent *****
// Handle global events, pass form-specific events to each form's event handler

static Boolean ApplicationHandleEvent(EventPtr event)
{
	FormType *pForm;
	UInt16 formID;
	Boolean handled = false;
	
	switch (event->eType) 
	{
		case frmLoadEvent:
		{
			formID = event->data.frmLoad.formID;
			pForm = FrmInitForm(formID);
			FrmSetActiveForm(pForm);

			switch (formID)
			{
				case jMileageForm:
					FrmSetEventHandler(pForm, jMileageFormEventHandler);
					break;

				case jMileageEditCategoriesForm:
					FrmSetEventHandler(pForm, EditCategoriesFormEventHandler);
					break;

				case jMileagePreferencesForm:
					FrmSetEventHandler(pForm, PreferencesFormEventHandler);
					break;

				case jMileageEditForm:
					FrmSetEventHandler(pForm, EditFormEventHandler);
					break;

				case jMileageDetailsForm:
					FrmSetEventHandler(pForm, DetailsFormEventHandler);
					break;
			}
			handled = true;
			break;
		}

		default:
			break;
	}

	return handled;
}








// ***** EventLoop *****
static void EventLoop()
{
	EventType event;
	UInt16 error;
	do
	{
		EvtGetEvent(&event, evtWaitForever);

		if (!SysHandleEvent(&event))
		{
			if (!MenuHandleEvent(0, &event, &error))
			{
				if (!ApplicationHandleEvent(&event))
				{
					FrmDispatchEvent(&event);
				}
			}
		}
	}
	while (event.eType != appStopEvent);
}






// ***** PilotMain *****
UInt32 PilotMain(UInt16 launchCode, MemPtr cmdPBP, UInt16 launchFlags)
{
	Err err = 0;

	// Check ROM version here

	if (launchCode == sysAppLaunchCmdNormalLaunch)
	{
		if ((err = StartApplication()) == 0)
		{
			EventLoop();
			StopApplication();
		}
	}
	return err;
}





// ***** jMRCompareFunc *****
// Compare function for jMileageRecordTypes, to determine sort order
Int16 jMRCompareFunc (void *rec1, void *rec2, Int16 other, 
	SortRecordInfoPtr rec1SortInfo,
	SortRecordInfoPtr rec2SortInfo, MemHandle appInfoH)
{
	if (((jMileageRecordType *)rec1)->startF < 
		((jMileageRecordType *)rec2)->startF) return -1;
	if (((jMileageRecordType *)rec1)->startF > 
		((jMileageRecordType *)rec2)->startF) return 1;
	return 0;
}







// ***** InitLabels *****
static void InitLabels()
{
	MemHandle resourceH, resource2H;
	Char *resource;
	Char *MPGString;
	Char *AMPGString;

	// free memory if not empty
	if (MPGStringH) MemHandleFree(MPGStringH);
	MPGStringH = NULL;
	if (AMPGStringH) MemHandleFree(AMPGStringH);
	AMPGStringH = NULL;

	// initialize resourceH and resource2H (depending on prefs.prefLabelType)
	if (prefs.prefLabelType==MPG)
	{
		resourceH = DmGetResource(strRsc, MPGStringR); // strRsc is a CONSTANT
		resource2H = DmGetResource(strRsc, AMPGStringR);
	}
	else if (prefs.prefLabelType==LP100KM)
	{
		resourceH = DmGetResource(strRsc, L100KmStringR);
		resource2H = DmGetResource(strRsc, AL100KmStringR);
	}
	else if (prefs.prefLabelType==KMPL)
	{
		resourceH = DmGetResource(strRsc, KmPLStringR);
		resource2H = DmGetResource(strRsc, AKmPLStringR);
	}
	// initialize MPGStringH
	resource = MemHandleLock(resourceH);
	MPGStringH = MemHandleNew(MemHandleSize(resourceH));
	MPGString = MemHandleLock(MPGStringH);
	StrCopy(MPGString, resource);
	MemHandleUnlock(MPGStringH);
	MemHandleUnlock(resourceH);
	DmReleaseResource(resourceH);
	// initialize AMPGStringH
	resource = MemHandleLock(resource2H);
	AMPGStringH = MemHandleNew(MemHandleSize(resource2H));
	AMPGString = MemHandleLock(AMPGStringH);
	StrCopy(AMPGString, resource);
	MemHandleUnlock(AMPGStringH);
	MemHandleUnlock(resource2H);
	DmReleaseResource(resource2H);
}







// ***** FillCarListArray *****
// This function fills the jMRTArrayH with elements from the DB from category
// prefs.carChosen-1.
static void FillCarListArray()
{
	jMileageRecordType *jMRTArray;
	UInt16 i, j, index;

	if (jMRTArrayH) MemHandleFree(jMRTArrayH);
	jMRTArrayNum = DmNumRecordsInCategory(dB, prefs.carChosen-1); // car chosen
	jMRTArrayH = MemHandleNew(jMRTArrayNum*sizeof(jMileageRecordType)
		+EXTRAINFO);
	jMRTArray = MemHandleLock(jMRTArrayH);
	MemSet(jMRTArray, MemHandleSize(jMRTArrayH), 0);
	for (i = 0, index = 0; i < jMRTArrayNum; i++)
	{
		MemHandle recordH;
		jMileageRecordType *recordP;
		jMileageRecordType *jMRT;
		recordH = DmQueryNextInCategory(dB, &index, prefs.carChosen-1);
		index ++;
		recordP = MemHandleLock(recordH);
		jMRT = jMRTArray + i;
		MemSet(jMRT, sizeof(jMileageRecordType), 0);
		jMRT->startF = recordP->startF;
		jMRT->endF = recordP->endF;
		jMRT->refillF = recordP->refillF;
		jMRT->date = recordP->date;
		jMRT->cpuF = recordP->cpuF;
		StrCopy(jMRT->notes, recordP->notes);
		MemHandleUnlock(recordH);
	}
	MemHandleUnlock(jMRTArrayH);
}






// ***** ExportData *****
// Exports the data to a memo for external synchronization.
static void ExportData()
{
	jMileageRecordType* jMRTArray;
	MemHandle mh;
	UInt16 bytesUsed;
	UInt16 i;
	UInt16 j;
	Char* str;
	Char tmpStr[100];

	// Instantiate mh and copy title
	mh = MemHandleNew(StrLen(EXPORT_TITLE_HEADER_STR) + 74*jMRTArrayNum 
		+ 1); // XXX 74 bytes per line
	str = MemHandleLock(mh);
	MemSet(str, sizeof(str)/sizeof(Char), 0);
	StrCopy(str, EXPORT_TITLE_HEADER_STR);
	bytesUsed = StrLen(EXPORT_TITLE_HEADER_STR);

	jMRTArray = MemHandleLock(jMRTArrayH);

	for (i = 0; i < jMRTArrayNum; i++)
	{
		jMileageRecordType *jMRT;
		DateTimeType dateTime;

		jMRT = jMRTArray + i;
		// Copy the date
		MemSet(tmpStr, sizeof(tmpStr)/sizeof(Char), 0);
		TimSecondsToDateTime(jMRT->date, &dateTime);
		DateToAscii(dateTime.month, dateTime.day, dateTime.year, dateFormatType, 
			tmpStr);
		StrCopy(str+bytesUsed, tmpStr);
		bytesUsed += StrLen(tmpStr);
		// Copy the ,
		StrCopy(str+bytesUsed, ",");
		bytesUsed += 1;
		// Copy the start mileage
		MemSet(tmpStr, sizeof(tmpStr)/sizeof(Char), 0);
		PrintF(tmpStr, jMRT->startF, 9, 1);
		StrCopy(str+bytesUsed, tmpStr);
		bytesUsed += StrLen(tmpStr);
		// Copy the ,
		StrCopy(str+bytesUsed, ",");
		bytesUsed += 1;
		// Copy the end mileage
		MemSet(tmpStr, sizeof(tmpStr)/sizeof(Char), 0);
		PrintF(tmpStr, jMRT->endF, 9, 1);
		StrCopy(str+bytesUsed, tmpStr);
		bytesUsed += StrLen(tmpStr);
		// Copy the ,
		StrCopy(str+bytesUsed, ",");
		bytesUsed += 1;
		// Copy the quantity
		MemSet(tmpStr, sizeof(tmpStr)/sizeof(Char), 0);
		PrintF(tmpStr, jMRT->refillF, 6, 3);
		StrCopy(str+bytesUsed, tmpStr);
		bytesUsed += StrLen(tmpStr);
		// Copy the ,
		StrCopy(str+bytesUsed, ",");
		bytesUsed += 1;
		// Copy the cost/quantity
		MemSet(tmpStr, sizeof(tmpStr)/sizeof(Char), 0);
		PrintF(tmpStr, jMRT->cpuF, 6, 3);
		StrCopy(str+bytesUsed, tmpStr);
		bytesUsed += StrLen(tmpStr);
		// Copy the ,
		StrCopy(str+bytesUsed, ",");
		bytesUsed += 1;
		// Copy the note
		MemSet(tmpStr, sizeof(tmpStr)/sizeof(Char), 0);
		StrCopy(tmpStr, jMRT->notes);
		//   replace all commas with space
		for (j = 0; j < StrLen(tmpStr); j++)
			if (tmpStr[j]==',') tmpStr[j]=' ';
		StrCopy(str+bytesUsed, tmpStr);
		bytesUsed += StrLen(jMRT->notes);
		// Copy the \n
		StrCopy(str+bytesUsed, "\n");
		bytesUsed += 1;
	}
	MemoCreateNewMemo(str);
	MemHandleUnlock(jMRTArrayH);
	MemHandleUnlock(mh);
	MemHandleFree(mh);

	MemSet(tmpStr, sizeof(tmpStr)/sizeof(Char), 0);
	StrPrintF(tmpStr, "%d", jMRTArrayNum);
	FrmCustomAlert(ExportCompleteAlert, tmpStr, NULL, NULL);
}






// ***** SaveFieldsToPrefs *****
// Saves fields from the form to prefs
// Return: error
static UInt8 SaveFieldsToPrefs(FormType *form)
{
	FieldType *field;
	Char *str;
	UInt8 error = 0;

	// save startF
	field = FrmGetObjectPtr(form, 
		FrmGetObjectIndex(form, jMileageStartField));
	str = FldGetTextPtr(field);
	if (str==NULL || StrLen(str)==0)
	{
		error = 1; // NULL startF
		prefs.startF[prefs.carChosen-1].f = -1;
	}
	else
	{
		// Parse start mileage into startF
		float startF = GccSysFlpAToF(str);
		if (startF < 0)
			error = 11; // invalid startF
		else
			prefs.startF[prefs.carChosen-1].f = startF;
	}

	// save endF
	field = FrmGetObjectPtr(form, 
		FrmGetObjectIndex(form, jMileageEndField));
	str = FldGetTextPtr(field);
	if (str==NULL || StrLen(str)==0)
	{
		error = 2; // NULL endF
		prefs.endF[prefs.carChosen-1].f = -1;
	}
	else
	{
		// Parse end mileage into endF
		float endF = GccSysFlpAToF(str);
		if (endF < 0)
			error = 12; // invalid endF
		else
			prefs.endF[prefs.carChosen-1].f = endF;
	}

	// save refillF
	field = FrmGetObjectPtr(form, 
		FrmGetObjectIndex(form, jMileageRefillField));
	str = FldGetTextPtr(field);
	if (str==NULL || StrLen(str)==0)
	{
		error = 3;
		prefs.refillF[prefs.carChosen-1].f = -1;
	}
	else
	{
		// Parse refill amt into refillF
		float refillF = GccSysFlpAToF(str);
		if (refillF < 0)
			error = 13; // invalid refillF
		else if (refillF == 0)
		{
			error = 3;
			prefs.refillF[prefs.carChosen-1].f = -1;
		}
		else
			prefs.refillF[prefs.carChosen-1].f = refillF;
	}

	// save cpuF
	field = FrmGetObjectPtr(form, 
		FrmGetObjectIndex(form, jMileageCPUField));
	str = FldGetTextPtr(field);
	if (str==NULL || StrLen(str)==0)
	{
		prefs.cpuF[prefs.carChosen-1].f = -1;
	}
	else
	{
		// Parse cost per unit amt into cpuF
		float cpuF = GccSysFlpAToF(str);
		if (cpuF < 0)
			error = 14; // invalid cpuF
		else if (cpuF == 0)
		{
			prefs.cpuF[prefs.carChosen-1].f = -1;
		}
		else
			prefs.cpuF[prefs.carChosen-1].f = cpuF;
	}

	// save notes
	field = FrmGetObjectPtr(form, 
		FrmGetObjectIndex(form, jMileageNoteField));
	str = FldGetTextPtr(field);
	if (str==NULL || StrLen(str)==0)
	{
		prefs.notes[prefs.carChosen-1][0] = 0;
	}
	else
	{
		// copy string into prefs.notes
		StrCopy(prefs.notes[prefs.carChosen-1], str);
	}
	return error;
}










// ***** FindIndexInDB *****
// Finds the location of jMRT in the DB via linear scan
Int16 FindIndexInDB(jMileageRecordType *jMRT)
{
	Int16 index = -1;
	MemHandle mh;
	jMileageRecordType *jMRT2;
	UInt16 i;
	Boolean found = false;

	for (i = 0; i < jMRTArrayNum; i++)
	{
		index ++;
		mh = DmQueryNextInCategory(dB, &index, prefs.carChosen-1);
		jMRT2 = MemHandleLock(mh);
		if (jMRT->startF == jMRT2->startF
			&& jMRT->endF == jMRT2->endF
			&& jMRT->refillF == jMRT2->refillF
			&& jMRT->date == jMRT2->date
			&& jMRT->cpuF == jMRT2->cpuF
			&& (StrCompare(jMRT->notes, jMRT2->notes)==0))
		{
			// this is the record we are looking for
			MemHandleUnlock(mh);
			found = true;
			break;
		}
		if (found) break;
		MemHandleUnlock(mh);
	}
	if (!found) index = -1;
	return index;
}






// ***** DeleteFromDB *****
// Delete the record with index index from the DB
static void DeleteFromDB(UInt16 index)
{
	// delete selected item
	DmDeleteRecord(dB, index);
	DmMoveRecord(dB, index, DmNumRecords(dB));
}





// ***** AddToDB *****
// Add record to the DB at index index
static void AddToDB(jMileageRecordType *jMRT)
{
	UInt16 size;
	UInt16 dBPos;
	MemHandle dBRecordH;
	jMileageRecordType *dBRecordP;
	UInt16 attributes;

	// calculate the size of jMRT
	size = sizeof(jMileageRecordType)-31+StrLen(jMRT->notes)+1;

	// save entry into database with appropriate category
	dBPos = DmFindSortPosition(dB, (MemPtr)jMRT, NULL,
		(DmComparF *)&jMRCompareFunc, 0);
	dBRecordH = DmNewRecord(dB, &dBPos, size);
	dBRecordP = MemHandleLock(dBRecordH);
	DmWrite(dBRecordP, 0, jMRT, size);
	MemHandleUnlock(dBRecordH);
	DmReleaseRecord(dB, dBPos, true);
	DmRecordInfo(dB, dBPos, &attributes, NULL, NULL);
	attributes &= ~dmRecAttrCategoryMask;
	attributes |= prefs.carChosen-1;
	attributes |= dmRecAttrDirty;
	DmSetRecordInfo(dB, dBPos, &attributes, NULL);
}










// ***** UpdateToV105 *****
// Update the db to version 105 by zeroing out unused fields and the prefs
// by zeroing out cpu and notes
static void UpdateToV105()
{
	UInt16 i, j;
	jMileageRecordType *jMRTArray;
	UInt8 sizeof_jMRT;

	// get the size of the record
	sizeof_jMRT = sizeof(jMileageRecordType) - 30*sizeof(Char);

	for (i = 0; i < DmNumRecordsInCategory(dB, dmAllCategories); i++)
	{
		MemHandle recordH;
		jMileageRecordType *jMRT, tmpRec;

		// get the record
		recordH = DmGetRecord(dB, i);
		if (recordH == NULL) 
			ErrDisplay("NULL record in UpdateToV105(1)");
		jMRT = MemHandleLock(recordH);

		// copy to temp record
		tmpRec = *jMRT;

		// unlock the record
		MemHandleUnlock(recordH);
		DmReleaseRecord(dB, i, true);

		// resize the record
		if (DmResizeRecord(dB, i, sizeof_jMRT)==NULL)
		{
			ErrAlert(ErrNoSpaceAlert);
			FrmCloseAllForms();
		}

		// reget and relock the record
		recordH = DmGetRecord(dB, i);
		if (recordH == NULL)
			ErrDisplay("NULL record in UpdateToV105(2)");
		jMRT = MemHandleLock(recordH);

		// update the temp record
		tmpRec.cpuF = -1;
		tmpRec.reserved = 0;
		for (j = 0; j < 31; j++)
			tmpRec.notes[j] = 0;

		// copy modified tmpRec into actual storage area
		if (DmWriteCheck(jMRT, 0, sizeof_jMRT) == dmErrWriteOutOfBounds)
			ErrDisplay("Write out of bounds");
		else if (DmWriteCheck(jMRT, 0, sizeof_jMRT) == dmErrNotValidRecord)
			ErrDisplay("Not valid record");

		DmWrite(jMRT, 0, &tmpRec, sizeof_jMRT);

		// release the record
		MemHandleUnlock(recordH);
		DmReleaseRecord(dB, i, true);
	}


	// update preferences
	//if (prefsRetrieved && prefs.car1[0] == 0) StrCopy(prefs.car1, "Auto 1");
	//if (prefsRetrieved && prefs.car2[0] == 0) StrCopy(prefs.car2, "Auto 2");
	//if (prefsRetrieved && prefs.car3[0] == 0) StrCopy(prefs.car3, "Auto 3");
	if (prefs.carChosen < 1 || prefs.carChosen > 3) prefs.carChosen = 1;
	if (prefs.prefLabelType <= 0 || prefs.prefLabelType > 3) 
		prefs.prefLabelType = MPG;
	if (prefs.prefDateType <= 0 || prefs.prefDateType > 4) 
		prefs.prefDateType = MSDSY;
	prefs.cpuF[0].f = prefs.cpuF[1].f = prefs.cpuF[2].f = -1;
	MemSet(&prefs.notes[0], sizeof(Char)*31, 0);
	MemSet(&prefs.notes[1], sizeof(Char)*31, 0);
	MemSet(&prefs.notes[2], sizeof(Char)*31, 0);
}







// ***** MemoCreateNewMemo *****
// Create a new memo for export.
static Boolean MemoCreateNewMemo(const Char* str)
{
	Boolean retval;
	UInt16 len;

	retval = false;

	if (str) 
	{
		len = StrLen((const Char *) str) + 1;
		if (str[0] && (len < 0x1000)) 
		{
			DmOpenRef dbref;

			dbref = DmOpenDatabaseByTypeCreator('DATA', 'memo', dmModeReadWrite);
			if (dbref) 
			{
				MemHandle h;
				UInt16  index;

				h = DmNewRecord(dbref, &index, len);
				if (h) 
				{
					MemPtr p;

					p = MemHandleLock(h);
					if (p) 
					{
						retval = true;

						if (DmWrite(p, 0, (MemPtr) str, len))
							retval = false;

						if (MemPtrUnlock(p))
							retval = false;
					}

					if (DmReleaseRecord(dbref, index, true)) 
					{
						retval = false;
					}

					if (!retval) 
					{
						DmDeleteRecord(dbref, index);
					}
				}

				if (DmCloseDatabase(dbref)) 
				{
					retval = false;
				}
			}
		}
	}

	return(retval);
}

