/******************************************************************************
 * File: jMileage.h
 * Application: jMileage
 * Revision 1.5
 * Copyright (c) 2001 Jason Shah, jasonshah@yahoo.com
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
*******************************************************************************/


// ***** Defines *****
#define EXTRAINFO 1 // Extra space needed on MemHandleNew
#define creatorID 	'jMPG'
#define dBType		'DATA'
#define dBName		"jMileageDB-jMPG"

//#define ERR_CHECK_LEVEL	ERR_CHECK_PARTIAL; // only fatal errors
#define ERR_CHECK_LEVEL	ERR_CHECK_FULL; // full errors, including nonfatals



// ***** Enums *****
enum ePrefLabelType { MPG, LP100KM, KMPL };
enum ePrefDateType { MSDSY, DPMPY, DSMSY, DDMDY };


// ***** Structures *****
// jMileagePrefType: items stored in preferences
typedef struct _jMileagePrefType
{
	FlpCompFloat startF[3];
	FlpCompFloat endF[3];
	FlpCompFloat refillF[3];
	UInt8 whichHasFocus[3]; // 1=startF, 2=endF, 3=refillF, 4=cpuF, 5=notes
	UInt8 carChosen; // which category (car) is chosen 1, 2, or 3
	Char car1[21];
	Char car2[21];
	Char car3[21];
	UInt8 prefLabelType; // 1=MPG, 2=L/100Km, 3=Km/L
	UInt8 prefDateType; // 1=m/d/y, 2=d.m.y, 3=d/m/y, 4=d-m-y
	UInt8 versionNum; // version of preferences stored, starting with 105 (1.5)
	FlpCompFloat cpuF[3]; // added in v1.5
	Char notes[3][31]; // added in v1.5
} jMileagePrefType;



// jMileageRecordType: records used in jMileage, stored/retrieved in DB
typedef struct _jMileageRecordType
{
	float startF;
	float endF;
	float refillF;
	UInt32 date;
	float cpuF; // cost per unit; added in v1.5
	UInt32 reserved; // for future use
	Char notes[31]; // first char of notes; notes end with '\0'; added in v1.5
		// Note: in actual dB, only chars up to \0 from notes are stored
} jMileageRecordType;



// jMileageAppInfoType: for app info block
typedef struct _jMileageAppInfoType
{
	// These six required for categories
	UInt16 renamedCategories;
	char categoryLabels[dmRecNumCategories][dmCategoryLength];
	UInt8 categoryUniqIDs[dmRecNumCategories];
	UInt8 lastUniqID;
	UInt8 reserved1;
	UInt16 reserved2;
} jMileageAppInfoType;



// ***** globals *****
static DmOpenRef dB; // database handle for reading/writing records
static MemHandle jMRTArrayH=0; // array of shown jMileageRecordTypes from list
static UInt16 jMRTArrayNum; // num of car records in jMRTArrayH
static MemHandle MPGStringH=NULL; // stores 'MPG' string
static MemHandle AMPGStringH=NULL; // stores 'Avg MPG' string
static UInt8 oldVersionNum;
static Char *carList[4]; // ptr to car list categories
static Char *carListLastOption = "Edit Categories";
static jMileagePrefType prefs; // stores prefs retrieved
static Boolean prefsRetrieved; // determine if the preferences were retrieved
static Int16 listSel; // set when Edit form called, to know which entry to edit
static Int16 month, day, year; // set on SelectDay, retrieved on OKBtn (Edit)
static DateFormatType dateFormatType; // from preferences
static UInt8 setFirstFocus; // value found in preferences
static Boolean endFSetFocus = false; // when true, sets focus to end field on
                                     // main list select (not start field)


// ***** Function headers *****
static Err StartApplication();
static void StopApplication();
Boolean jMileageFormEventHandler(EventPtr event);
Boolean EditCategoriesFormEventHandler(EventPtr event);
Boolean PreferencesFormEventHandler(EventPtr event);
Boolean EditFormEventHandler(EventPtr event);
Boolean DetailsFormEventHandler(EventPtr event);
static void DrawAMPG();
static void DrawMPG(UInt8 selection);
static void ClearAllMPG();
static void ClearMPG();
static void ListDrawFn(UInt16 itemNum, RectangleType *bounds,
	Char **itemsText);
static Boolean ApplicationHandleEvent(EventPtr event);
static void EventLoop();
UInt32 PilotMain(UInt16 launchCode, MemPtr cmdPBP, UInt16 launchFlags);
Int16 jMRCompareFunc (void *rec1, void *rec2, Int16 other, 
	SortRecordInfoPtr rec1SortInfo,
	SortRecordInfoPtr rec2SortInfo, MemHandle appInfoH);
static void InitLabels();
static void FillCarListArray();
static void ExportData();
static UInt8 SaveFieldsToPrefs(FormType *form);
static Int16 FindIndexInDB(jMileageRecordType *jMRT);
static void DeleteFromDB(UInt16 index);
static void AddToDB(jMileageRecordType *jMRT);
static void UpdateToV105();
static Boolean MemoCreateNewMemo(const Char* str);

